==> maxdb-source-7_6_00_09.tgz: V76_00_09/MaxDB_ORG/sys/src/SAPDB/SQLManager/Query/Query_ManipulationContext.hpp <==
/*!***************************************************************************

  module      : Query_ManipulationContext.hpp

  -------------------------------------------------------------------------

  responsible : DirkT

  special area: SQL Manager / Query
  description : 

  last changed: 2004-11-23  16:00
  first created:2004-04-29  10:00

  -------------------------------------------------------------------------

  copyright:    Copyright (c) 2000-2004 SAP AG




    ========== licence begin  GPL
    Copyright (c) 2000-2004 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end



*****************************************************************************/

#ifndef QUERY_MANIPULATIONCONTEXT_H
#define QUERY_MANIPULATIONCONTEXT_H

// ------------------------------------------------------------------------------------------------

#include "SAPDB/SAPDBCommon/SAPDB_Types.hpp"
#include "SAPDB/SQLManager/Query/Query_Types.hpp"

#include "SAPDB/KernelCommon/Kernel_VTrace.hpp"

#include "hak01.h"
#include "hgg01_3.h"
#include "hsp30.h"

// ------------------------------------------------------------------------------------------------

class SQLMan_Context;
class SAPDBMem_IRawAllocator;
class SharedSQL_PrepareHandle;

// ------------------------------------------------------------------------------------------------

class Query_ManipulationContext
{
public:
    Query_ManipulationContext( SQLMan_Context& SQLManContext, SharedSQL_PrepareHandle* prepareHandle );

    ~Query_ManipulationContext( void );

    inline bool KeepChanges( void )
    {
        SAPDBERR_ASSERT_STATE( !mDecision );

        mDecision = true;
        return KeepNewSyntaxTreeAndStatementBuffer();
    }

    inline bool DiscardChanges( void )
    {
        SAPDBERR_ASSERT_STATE( !mDecision );

        mDecision = true;
        return RestoreOldSyntaxTreeAndStatementBuffer();
    }

    inline bool IsOk( void )
    {
        return !mError;
    }

    void SetError( void )
    {
        mError = true;
    }

    SAPDBMem_IRawAllocator& GetAllocator( void ) const;
    void* GetSQLStatementPtr( void ) const;

	SharedSQL_PrepareHandle* GetPrepareHandle( void ) const { return mPrepareHandle; }

    int GetSQLStatementSize( void ) const;
    bool AppendBlankToSQLStatement(void);
    SAPDB_Int4 AppendToSQLStatement( char* ToAdd );
    SAPDB_Int4 AppendToSQLStatement( void* ToAdd, SAPDB_Int4 AddSize );
    bool ExtendSQLStatementBuffer( SAPDB_Int4 RequiredSize );
    inline SAPDB_Int4 FindSubStrInStatementBuffer( SAPDB_Byte* SubStr, const SAPDB_Int2 Len ) const;
    inline void       ReplaceSubStrInStatementBuffer( const SAPDB_Int4 Pos, SAPDB_Byte* SubStr, const SAPDB_Int2 Len );
    inline SAPDB_Int2 RTrimLen( SAPDB_Byte* Str, const SAPDB_Int2 Len );

    bool Changes( void ) const;

    inline SQLMan_Context& SQLManContext( void )
    {
        return mSQLManContext;
    }

    inline Kernel_VTrace& VTrace( void )
    {
        return mVTrace;
    }

protected:
    void CreateNewSyntaxTreeAndStatementBuffer( void );
    bool RestoreOldSyntaxTreeAndStatementBuffer( void );
    bool KeepNewSyntaxTreeAndStatementBuffer( void );

private:
    SQLMan_Context&     mSQLManContext;

	SharedSQL_PrepareHandle* mPrepareHandle;

    Kernel_VTrace       mVTrace;

    tak_ap_node*        mOldAPTreePtr;
    SyntaxTreeNodeNo    mOldScvIndex;
    SyntaxTreeNodeNo    mOldMaxNode;
    SyntaxTreeNodeNo    mOldAPMaxNode;
    SyntaxTreeNodeNo    mOldFirstHintNode;

    SAPDB_Byte*         mSQLStatementBuffer;
    SAPDB_Int4          mSQLStatementBufferMaxSize;
    SAPDB_Int4          mSQLStatementBufferUsedSize;
    bool                mSQLStatementBufferChanges;

    bool                mError;
    bool                mDecision;
    bool				mChanges;
};

// ------------------------------------------------------------------------------------------------

inline SAPDB_Int2 Query_ManipulationContext::RTrimLen( SAPDB_Byte* Str, const SAPDB_Int2 Len )
{
    return s30lnr_defbyte(Str, (g01unicode?csp_unicode_def_byte:bsp_c1), 0, Len);
}

inline SAPDB_Int4 Query_ManipulationContext::FindSubStrInStatementBuffer( SAPDB_Byte* SubStr, const SAPDB_Int2 Len ) const
{
    SAPDB_Int4 Pos = 0;
    SAPDB_Byte* Buffer = mSQLStatementBuffer;
    
    while ( Pos <= mSQLStatementBufferUsedSize-Len )
    {
        SAPDB_Byte* tmpsubstr = (SAPDB_Byte*) SubStr;
        SAPDB_Byte* tmpstr    = Buffer;
        SAPDB_Int2 i = 0;
        while ( (i<Len) && (*tmpstr++ == *tmpsubstr++) ) { ++i; }
        if (i == Len)
            return Pos;

        ++Buffer;
        ++Pos;
    }
    return -1;
}

inline void Query_ManipulationContext::ReplaceSubStrInStatementBuffer( const SAPDB_Int4 Pos, SAPDB_Byte* SubStr, const SAPDB_Int2 Len )
{
    SAPDB_Byte* Dest = mSQLStatementBuffer + Pos;
    SAPDB_Byte* Source = (SAPDB_Byte*) SubStr;

    SAPDB_Int2 i = 0;
    while ( (i++<Len) && (*Dest++ = *Source++) ) {}
}

#endif  // QUERY_MANIPULATIONCONTEXT_H

==> maxdb-source-7_6_00_27.tgz: V76_00_27/MaxDB_ORG/sys/src/SAPDB/SQLManager/Query/Query_ManipulationContext.hpp <==
/*!***************************************************************************

    @file     Query_ManipulationContext.hpp
    @ingroup  Query
    @author   DirkT
    @brief    
    @see  

  -------------------------------------------------------------------------

  responsible : DirkT

  special area: SQL Manager / Query
  description : 

  last changed: 2005-06-08  12:00
  first created:2004-04-29  10:00

  -------------------------------------------------------------------------

  copyright:    Copyright (c) 2000-2005 SAP AG




    ========== licence begin  GPL
    Copyright (c) 2000-2005 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end




*****************************************************************************/

#ifndef QUERY_MANIPULATIONCONTEXT_H
#define QUERY_MANIPULATIONCONTEXT_H

// ------------------------------------------------------------------------------------------------

#include "SAPDB/SAPDBCommon/SAPDB_Types.hpp"
#include "SAPDB/SQLManager/Query/Query_Types.hpp"

#include "SAPDB/KernelCommon/Kernel_VTrace.hpp"

#include "hak01.h"
#include "hgg01_3.h"
#include "hsp30.h"

// ------------------------------------------------------------------------------------------------

class SQLMan_Context;
class SAPDBMem_RawAllocator;
class SharedSQL_PrepareHandle;

// ------------------------------------------------------------------------------------------------
/*!
    @class          Query_ManipulationContext
    @brief          ...
 */
class Query_ManipulationContext
{
public:
    /// ...
    Query_ManipulationContext( SQLMan_Context& SQLManContext, SharedSQL_PrepareHandle* prepareHandle );

    /// ...
    ~Query_ManipulationContext( void );

    /// ...
    inline bool KeepChanges( void )
    {
        SAPDBERR_ASSERT_STATE( !mDecision );

        mDecision = true;
        return KeepNewSyntaxTreeAndStatementBuffer();
    }

    /// ...
    inline bool DiscardChanges( void )
    {
        SAPDBERR_ASSERT_STATE( !mDecision );

        mDecision = true;
        return RestoreOldSyntaxTreeAndStatementBuffer();
    }

    /// ...
    inline bool IsOk( void )
    {
        return (!mError && !mNotYetImplemented);
    }

    /// ...
    void SetError( void )
    {
        mError = true;
    }

    /// ...
    void SetNotYetImplemented( void )
    {
        mNotYetImplemented = true;
    }

    /// ...
    SAPDBMem_RawAllocator& GetAllocator( void ) const;
    /// ...
    void* GetSQLStatementPtr( void ) const;

    /// ...
	SharedSQL_PrepareHandle* GetPrepareHandle( void ) const { return mPrepareHandle; }

    /// ...
    int GetSQLStatementSize( void ) const;
    /// ...
    bool AppendBlankToSQLStatement(void);
    /// ...
    SAPDB_Int4 AppendToSQLStatement( char* ToAdd );
    /// ...
    SAPDB_Int4 AppendToSQLStatement( void* ToAdd, SAPDB_Int4 AddSize );
    /// ...
    bool ExtendSQLStatementBuffer( SAPDB_Int4 RequiredSize );

    /// ...
    inline SQLMan_Context& SQLManContext( void )
    {
        return mSQLManContext;
    }

    /// ...
    inline Kernel_VTrace& VTrace( void )
    {
        return mVTrace;
    }

protected:
    /// ...
    bool CreateNewSyntaxTreeAndStatementBuffer( void );
    /// ...
    bool RestoreOldSyntaxTreeAndStatementBuffer( void );
    /// ...
    bool KeepNewSyntaxTreeAndStatementBuffer( void );

private:
    /// ...
    SQLMan_Context&     mSQLManContext;   
	/// ...
	SAPDBMem_RawAllocator* mRawAllocator;

    /// ...
	SharedSQL_PrepareHandle* mPrepareHandle;            

    /// ...
    Kernel_VTrace       mVTrace;                        

    /// ...
	void*				mOldContextStmtPart;            

    /// ...
    SAPDB_Byte*         mSQLStatementBuffer;            
    /// ...
    SAPDB_Int4          mSQLStatementBufferMaxSize;     
    /// ...
    SAPDB_Int4          mSQLStatementBufferUsedSize;    
    /// ...
    bool                mSQLStatementBufferChanges;     

    /// ...
    bool                mError;                         
    /// ...
	bool                mNotYetImplemented;             
    /// ...
    bool                mDecision;     
#if defined( QRW_FIND_ROLLBACK_BY_ERROR )
    ///
    SAPDB_ULong         mSavedBytesUsed;
#endif
};

// ------------------------------------------------------------------------------------------------

#endif  // QUERY_MANIPULATIONCONTEXT_H
